import os, rox, gtk, gc
from rox import loading
from debug_utils import *

# Constants for scaling.
SCALE_NORM		= 0
SCALE_UP		= 1
SCALE_DOWN		= -1
SCALE_FIT		= 2
RATIO_NORMAL	= 1.0
RATIO_FIT			= -1.0
RATIO_STEP		= 0.1

class ImageLogic(loading.XDSLoader):
	def __init__(self):
		loading.XDSLoader.__init__(self, None)
		self.imgs = []
		self.fit_w = 10
		self.fit_h = 10
		self.idx = -1
		self.setting_image = False
		self.thumbnails = []
		self.pixbuf = None
		self.width, self.height = 0, 0

	def xds_load_from_file(self, path):
		''' Loads in DnDed path. '''
		self.append(path)
		self.update_window()

	def append(self, filename):
		''' Takes in filename. Adds all images found within. '''
		filename = os.path.abspath(filename)
		self.set_title('Loading ' + filename + '...')
		gtk.main_iteration()

		if os.path.isdir(filename):
			# Load all files in directory.
			for fn in os.listdir(filename):
				checkfn = os.path.join(filename, fn);
				if os.path.isdir(checkfn):
					if self.recursive_load.int_value:
						self.append(checkfn)
				else:
					self.append(checkfn)
			return

		if not os.path.exists(filename):
			return False

		if filename in self.imgs:
			# Set idx to dropped image, update
			self.idx = self.imgs.index(filename)
			self.show_after_append()
			return

		type = str(rox.mime.get_type(filename)).split('/')[0]
		if type == 'image':
			try:
				pixbuf = gtk.gdk.pixbuf_new_from_file(filename)
				thumbnail = self.scale_to_size(pixbuf, 30)
				self.imgs.append(filename)
				self.thumbnails.append(thumbnail)
				del pixbuf
				del thumbnail
			except:
				# Can't load the image. Image is probably broken, just ignore it.
				pass
			gc.collect()

			#Show the first image as soon as possible
			if self.idx < 0:
				self.set_idx(0)
				# self.update_image()

	def set_idx(self, idx):
		''' Sets self.idx to idx, and updates the UI'''
		if not self.sensitive:
			return

		self.idx = idx
		self.make_idx_sane()

		self.update_image()

		if ((self.resize_on_load.value == 'Always') or
			(self.resize_on_load.value == 'Startup' and self.startup)):
			self.resize_to_pixbuf(self.pixbuf)

		self.update_scrollbars()
		self.update_window()

	def scale_to_size(self, pixbuf, size):
		''' Returns a pixmap scaled in width and height to, at most, 'size.' '''
		size_h = float(pixbuf.get_height())
		size_w = float(pixbuf.get_width())
		ratio = float(1)
		size = float(size)
		if size_h > size_w:
			# Height larger.
			ratio = float(size_h / size)
		else:
			# Sizes equal, or width larger
			ratio = float(size_w / size)
		if ratio <= 0.0:
			ratio = 1.0
		size_h, size_w = int(size_h / ratio), int(size_w / ratio)
		return pixbuf.scale_simple(size_w, size_h, gtk.gdk.INTERP_BILINEAR)

	def scale_to_ratio(self, pixbuf, ratio):
		'''Returns pixbuf scaled to ratio... 'ratio' '''
		size_h = int(pixbuf.get_height() * ratio)
		size_w = int(pixbuf.get_width() * ratio)
		return pixbuf.scale_simple(size_w, size_h, gtk.gdk.INTERP_BILINEAR)

	def scale_to_fit(self, pixbuf):
		''' Updates displayed image to scaled version of self.pixbuf '''
		if self.fit_w < 10 or self.fit_h < 10:
			return

		img_w = pixbuf.get_width()
		img_h = pixbuf.get_height()

		size_w = float(self.fit_w)
		size_h = float(self.fit_h)

		resize_w = size_w / img_w
		resize_h = size_h / img_h

		if img_h * resize_w > size_h:
			#resize_w = (img_h * resize_w) / size_h
			size = resize_h
		elif img_w * resize_h > size_w:
			#resize_h = (img_w * resize_h) / size_w
			size = resize_w
		elif resize_w < resize_h:
			size = resize_w
		else:
			size = resize_h

		# debugger('Setting image to size %f'%size)
		return self.scale_to_ratio(pixbuf, size)

	def resize_event(self, widget, rec = None):
		''' Called when the window is resized. Should update the image's scaling accordingly.'''
		if self.ratio != -1:
			return
		
		width, height = rec.width, rec.height
		
		if self.fit_w == width and self.fit_h == height:
			# debugger('nothing to scale')
			return

		self.fit_h = height
		self.fit_w = width

		# Make them sane...
		if self.fit_h < 10:
			self.fit_h = 10
		if self.fit_w < 10:
			self.fit_w = 10

		# debugger("fit_size w %d h %d"%(self.fit_w, self.fit_h))
		self.update_image()

	def resize_to_pixbuf(self, pixbuf):
		''' Resizes the window if options are OK with it, or dies trying.'''
		if self.ratio == RATIO_FIT:
			# We fit image to window, no point in resizing. ;)
			return

		width = pixbuf.get_width()
		height = pixbuf.get_height()
		tool_h = self.toolbar.get_child_requisition()[1]
		# debugger('image w %d h %d - toolbar h %d'%(width, height, tool_h))

		# Add the toolbar's size, and add some extra pixels to avoid the scrollbars.
		width += 8
		height += (tool_h + 8)
		screen_w, screen_h = (gtk.gdk.screen_width() / 4) * 3, (gtk.gdk.screen_height() / 4) * 3
		# debugger('image w %d h %d - screen w %d h %d'%(width, height, screen_w, screen_h))

		if width > screen_w: width = screen_w
		if height > screen_h: height = screen_h

		self.resize(width, height)
		self.move_window_onscreen(width, height)

	def move_window_onscreen(self, size_x, size_y):
		''' Moves the window back within screen bounds, assuming it isn't already.'''
		BOARDER = 40
		loc_x, loc_y = self.get_position()
		x, y = gtk.gdk.screen_width(), gtk.gdk.screen_height()
		move_x, move_y = None, None

		if loc_x + size_x > x:
			move_x = x - size_x - BOARDER
		if loc_y + size_y > y:
			move_y = y - size_y - BOARDER

		if move_x or move_y:
			if not move_x:
				move_x = loc_x
			if not move_y:
				move_y = loc_y
			# debugger('x %d - y %d'%(move_x, move_y))
			self.move(move_x, move_y)

	def make_idx_sane(self):
		''' Makes self.idx sane.'''
		if self.idx >= len(self.imgs):
			self.idx = 0
		if self.idx < 0:
			self.idx = len(self.imgs) - 1

	def make_image(self, filename):
		''' Attempts to return a Pixbuf of 'filename.' Does not sanity check!'''
		pixbuf = gtk.gdk.pixbuf_new_from_file(filename)
		if self.ratio == RATIO_FIT:
			pixbuf = self.scale_to_fit(pixbuf)
		elif self.ratio != RATIO_NORMAL:
			assert self.ratio > 0.0
			pixbuf = self.scale_to_ratio(pixbuf, self.ratio)
		return pixbuf

	def update_scrollbars(self):
		''' Show/hide scrollbars, and position them at 0, 0'''

		if self.ratio == RATIO_FIT:
			self.image.set_size_request(10, 10)
			self.scrolled_window.set_policy(gtk.POLICY_NEVER, gtk.POLICY_NEVER)
		else:
			self.image.set_size_request(-1, -1)
			self.scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)

		# Set scroll bars to 0,0.
		for adj in [self.scrolled_window.get_hadjustment(), self.scrolled_window.get_vadjustment()]:
			adj.set_value(0.0)

	def update_image(self):
		''' Sets image held in idx.'''
		if not len(self.imgs):
			return

		# self.make_idx_sane()
		if not os.path.exists(self.imgs[self.idx]):
			# Deleted image. Remove from filelist.
			del self.imgs[self.idx]
			self.update_image()
		
		if self.pixbuf:
			del self.pixbuf

		self.pixbuf = self.make_image( self.imgs[self.idx] )
		self.image.set_from_pixbuf(self.pixbuf)

		# Nudge gc to eat uneeded pixbuf[s],
		gc.collect()
